@@ -1,4 +1,4 @@
-export CC=gcc
-export CXX=g++
+export CC=clang
+export CXX=clang++
 export FC=gfortran
 export F77=gfortran
