@@ -9,10 +9,6 @@
 
 #ifdef WITH_SLEPC
 
-#if PETSC_VERSION_LT(3, 15, 0)
-#define SVDSetOperators(a, b, c) SVDSetOperator(a, b)
-#endif
-
 #include "slepc.h"
 
 namespace SLEPc {
@@ -105,7 +101,7 @@ class eigensolver : public OneOperator {
                 Expression B;
                 const OneOperator* codeA, *codeB;
                 const int c;
-                static const int n_name_param = 10;
+                static const int n_name_param = 11;
                 static basicAC_F0::name_and_type name_param[];
                 Expression nargs[n_name_param];
                 E_eigensolver(const basicAC_F0& args, int d) : A(0), B(0), codeA(0), codeB(0), c(d) {
@@ -155,9 +151,10 @@ basicAC_F0::name_and_type eigensolver<Type, K, SType>::E_eigensolver::name_param
     {!std::is_same<SType, SVD>::value ? "array" : "larray", &typeid(KNM<K>*)},
     {"fields", &typeid(KN<double>*)},
     {"names", &typeid(KN<String>*)},
-    {!std::is_same<SType, SVD>::value ? "schurPreconditioner" : "rvectors", !std::is_same<SType, SVD>::value ? &typeid(KN<Matrice_Creuse<HPDDM::upscaled_type<PetscScalar>>>*) : &typeid(FEbaseArrayKn<K>*)},
+    {!std::is_same<SType, SVD>::value ? "schurPreconditioner" : "rvectors", !std::is_same<SType, SVD>::value ? &typeid(KN<Matrice_Creuse<upscaled_type<PetscScalar>>>*) : &typeid(FEbaseArrayKn<K>*)},
     {!std::is_same<SType, SVD>::value ? "schurList" : "rarray", !std::is_same<SType, SVD>::value ? &typeid(KN<double>*) : &typeid(KNM<K>*)},
     {"deflation", &typeid(KNM<PetscScalar>*)},
+    {"errorestimate", &typeid(KN<PetscReal>*)},
 };
 template<class Type, class K, class SType>
 struct _n_User {
@@ -177,7 +174,7 @@ PetscErrorCode FormFun(NEP nep, PetscScalar lambda, Mat F, Mat P, void* ctx) {
         reinterpret_cast< typename eigensolver<PETSc::DistributedCSR<HpSchwarz<PetscScalar>>, PetscScalar, NEP>::ScalarF_O* >((*user)->F);
     int ret;
     f->apply(lambda, &ret);
-    PetscFunctionReturn(ret);
+    PetscFunctionReturn(PetscErrorCode(ret));
 }
 PetscErrorCode FormJac(NEP nep, PetscScalar lambda, Mat J, void* ctx) {
     User<PETSc::DistributedCSR<HpSchwarz<PetscScalar>>, PetscScalar, NEP>* user;
@@ -188,7 +185,7 @@ PetscErrorCode FormJac(NEP nep, PetscScalar lambda, Mat J, void* ctx) {
         reinterpret_cast< typename eigensolver<PETSc::DistributedCSR<HpSchwarz<PetscScalar>>, PetscScalar, NEP>::ScalarF_O* >((*user)->J);
     int ret;
     f->apply(lambda, &ret);
-    PetscFunctionReturn(ret);
+    PetscFunctionReturn(PetscErrorCode(ret));
 }
 template<class Type, class K, class SType>
 AnyType eigensolver<Type, K, SType>::E_eigensolver::operator()(Stack stack) const {
@@ -274,12 +271,16 @@ AnyType eigensolver<Type, K, SType>::E_eigensolver::operator()(Stack stack) cons
                 else
                     PEPSetOptionsPrefix(pep, GetAny<std::string*>((*nargs[1])(stack))->c_str());
             }
+            KSP empty = NULL;
             if(std::is_same<SType, EPS>::value) {
                 EPSSetFromOptions(eps);
                 ST st;
                 EPSGetST(eps, &st);
-                if(ptA->_ksp)
+                if(ptA->_ksp) {
+                    STGetKSP(st, &empty);
+                    PetscObjectReference((PetscObject)empty);
                     STSetKSP(st, ptA->_ksp);
+                }
                 else {
                     KSP ksp;
                     PC pc;
@@ -293,7 +294,7 @@ AnyType eigensolver<Type, K, SType>::E_eigensolver::operator()(Stack stack) cons
                     if(isFieldSplit) {
                         KN<double>* fields = nargs[5] ? GetAny<KN<double>*>((*nargs[5])(stack)) : 0;
                         KN<String>* names = nargs[6] ? GetAny<KN<String>*>((*nargs[6])(stack)) : 0;
-                        KN<Matrice_Creuse<HPDDM::upscaled_type<PetscScalar>>>* mS = nargs[7] ? GetAny<KN<Matrice_Creuse<HPDDM::upscaled_type<PetscScalar>>>*>((*nargs[7])(stack)) : 0;
+                        KN<Matrice_Creuse<upscaled_type<PetscScalar>>>* mS = nargs[7] ? GetAny<KN<Matrice_Creuse<upscaled_type<PetscScalar>>>*>((*nargs[7])(stack)) : 0;
                         KN<double>* pL = nargs[8] ? GetAny<KN<double>*>((*nargs[8])(stack)) : 0;
                         if(fields && names) {
                             KSP ksp;
@@ -389,6 +390,7 @@ AnyType eigensolver<Type, K, SType>::E_eigensolver::operator()(Stack stack) cons
                 PEPGetConverged(pep, &nconv);
             if(nconv > 0 && ((nargs[2] || nargs[3] || nargs[4]) || (std::is_same<SType, SVD>::value && (nargs[7] || nargs[8])))) {
                 KN<typename std::conditional<!std::is_same<SType, SVD>::value, K, PetscReal>::type>* eigenvalues = nargs[2] ? GetAny<KN<typename std::conditional<!std::is_same<SType, SVD>::value, K, PetscReal>::type>*>((*nargs[2])(stack)) : nullptr;
+                KN<PetscReal>* errorestimate = nargs[10] ? GetAny<KN<PetscReal>*>((*nargs[10])(stack)) : nullptr;
                 KNM<K>* array = nargs[4] ? GetAny<KNM<K>*>((*nargs[4])(stack)) : nullptr;
                 FEbaseArrayKn<K>* rvectors = std::is_same<SType, SVD>::value && nargs[7] ? GetAny<FEbaseArrayKn<K>*>((*nargs[7])(stack)) : nullptr;
                 KNM<K>* rarray = std::is_same<SType, SVD>::value && nargs[8] ? GetAny<KNM<K>*>((*nargs[8])(stack)) : nullptr;
@@ -398,6 +400,8 @@ AnyType eigensolver<Type, K, SType>::E_eigensolver::operator()(Stack stack) cons
                     eigenvectors->resize(nconv);
                 if(rvectors && !isType)
                     rvectors->resize(nconv);
+                if(errorestimate)
+                    errorestimate->resize(nconv);
                 if(array)
                     array->resize(m, nconv);
                 Vec xr, xi;
@@ -412,14 +416,24 @@ AnyType eigensolver<Type, K, SType>::E_eigensolver::operator()(Stack stack) cons
                 for(PetscInt i = 0; i < nconv; ++i) {
                     PetscScalar kr, ki = 0;
                     PetscReal sigma;
-                    if(std::is_same<SType, EPS>::value)
+                    PetscReal errest;
+                    if(std::is_same<SType, EPS>::value) {
                         EPSGetEigenpair(eps, i, &kr, &ki, (eigenvectors || array) ? xr : NULL, (eigenvectors || array) && std::is_same<PetscScalar, double>::value && std::is_same<K, std::complex<double>>::value ? xi : NULL);
+                        if(errorestimate)
+                            EPSGetErrorEstimate(eps, i, &errest);
+                    }
                     else if(std::is_same<SType, SVD>::value)
                         SVDGetSingularTriplet(svd, i, &sigma, xr, xi);
-                    else if(std::is_same<SType, NEP>::value)
+                    else if(std::is_same<SType, NEP>::value) {
                         NEPGetEigenpair(nep, i, &kr, &ki, (eigenvectors || array) ? xr : NULL, (eigenvectors || array) && std::is_same<PetscScalar, double>::value && std::is_same<K, std::complex<double>>::value ? xi : NULL);
-                    else
+                        if(errorestimate)
+                            NEPGetErrorEstimate(nep, i, &errest);
+                    }
+                    else {
                         PEPGetEigenpair(pep, i, &kr, &ki, (eigenvectors || array) ? xr : NULL, (eigenvectors || array) && std::is_same<PetscScalar, double>::value && std::is_same<K, std::complex<double>>::value ? xi : NULL);
+                        if(errorestimate)
+                            PEPGetErrorEstimate(pep, i, &errest);
+                    }
                     if(eigenvectors || array || rvectors || rarray) {
                         PetscScalar* tmpr;
                         PetscScalar* tmpi;
@@ -437,6 +451,7 @@ AnyType eigensolver<Type, K, SType>::E_eigensolver::operator()(Stack stack) cons
                         else {
                             pt = reinterpret_cast<K*>(tmpr);
                             VecGetArray(xi, reinterpret_cast<PetscScalar**>(&pti));
+                            tmpi = reinterpret_cast<PetscScalar*>(pti);
                         }
                         if(!isType && (ptA->_A || ptA->_exchange)) {
                             KN<K> cpy(ptA->_A ? ptA->_A->getDof() : ptA->_exchange[0]->getDof());
@@ -485,7 +500,7 @@ AnyType eigensolver<Type, K, SType>::E_eigensolver::operator()(Stack stack) cons
                         }
                         if(!std::is_same<SType, SVD>::value && std::is_same<PetscScalar, double>::value && std::is_same<K, std::complex<double>>::value)
                             delete [] pt;
-                        else
+                        if(std::is_same<SType, SVD>::value || (std::is_same<PetscScalar, double>::value && std::is_same<K, std::complex<double>>::value))
                             VecRestoreArray(xi, &tmpi);
                         VecRestoreArray(xr, &tmpr);
                     }
@@ -495,6 +510,8 @@ AnyType eigensolver<Type, K, SType>::E_eigensolver::operator()(Stack stack) cons
                         else
                             eigenvalues->operator[](i) = sigma;
                     }
+                    if(errorestimate)
+                        errorestimate->operator[](i) = errest;
                 }
                 if(eigenvectors || array || rvectors || rarray) {
                     VecDestroy(&xr);
@@ -506,8 +523,15 @@ AnyType eigensolver<Type, K, SType>::E_eigensolver::operator()(Stack stack) cons
                 delete user->mat;
                 PetscFree(user);
             }
-            if(std::is_same<SType, EPS>::value)
+            if(std::is_same<SType, EPS>::value) {
+                if(empty) {
+                    ST st;
+                    EPSGetST(eps, &st);
+                    STSetKSP(st, empty);
+                    PetscObjectDereference((PetscObject)empty);
+                }
                 EPSDestroy(&eps);
+            }
             else if(std::is_same<SType, SVD>::value)
                 SVDDestroy(&svd);
             else if(std::is_same<SType, NEP>::value)
@@ -539,7 +563,7 @@ static PetscErrorCode MatMult_User(Mat A, Vec x, Vec y) {
     yy = *mat * xx;
     VecRestoreArray(y, &out);
     VecRestoreArrayRead(x, &in);
-    PetscFunctionReturn(0);
+    PetscFunctionReturn(PETSC_SUCCESS);
 }
 void finalizeSLEPc() {
     SlepcFinalize();
