@@ -1,6 +1,8 @@
 #ifndef BEM_HPP_
 #define BEM_HPP_
 
+#include <htool/htool.hpp>
+
 #if defined(WITH_metis)
 extern "C" {
 #include <metis.h>
@@ -72,7 +74,7 @@ KN< R > *partmetis( KN< R > *const &part, const FESPACE * pVh, long const &lpart
 #endif
 
 double ff_htoolEta=10., ff_htoolEpsilon=1e-3;
-long ff_htoolMinclustersize=10, ff_htoolMaxblocksize=1000000, ff_htoolMintargetdepth=0, ff_htoolMinsourcedepth=0;
+long ff_htoolMinclustersize=10, ff_htoolMintargetdepth=0, ff_htoolMinsourcedepth=0;
 
 template<class K>
 class HMatrixVirt {
@@ -88,31 +90,12 @@ public:
     virtual int get_sizeworld() const = 0;
     virtual int get_local_size() const = 0;
     virtual int get_local_offset() const = 0;
-    virtual const std::vector<htool::SubMatrix<K>*>& get_MyNearFieldMats() const = 0;
-    virtual const htool::LowRankMatrix<K>& get_MyFarFieldMats(int i) const = 0;
-    virtual int get_MyFarFieldMats_size() const = 0;
-    virtual const std::vector<htool::SubMatrix<K>*>& get_MyStrictlyDiagNearFieldMats() const = 0;
+    virtual std::vector<const htool::HMatrix<K>*> get_dense_blocks() const = 0;
+    virtual std::vector<const htool::HMatrix<K>*> get_low_rank_blocks() const = 0;
+    virtual std::vector<const htool::HMatrix<K>*> get_diagonal_blocks() const = 0;
     virtual htool::Matrix<K> get_local_dense() const = 0;
     virtual int get_permt(int) const = 0;
     virtual int get_perms(int) const = 0;
-    virtual char get_symmetry_type() const = 0;
-    virtual void set_compression(std::shared_ptr<htool::VirtualLowRankGenerator<K>> compressor) = 0;
-
-    // Getters/setters for parameters
-    virtual double get_epsilon() const                            = 0;
-    virtual double get_eta() const                                = 0;
-    virtual int get_minsourcedepth() const                        = 0;
-    virtual int get_mintargetdepth() const                        = 0;
-    virtual int get_maxblocksize() const                          = 0;
-    virtual void set_epsilon(double epsilon0)                     = 0;
-    virtual void set_eta(double eta0)                             = 0;
-    virtual void set_minsourcedepth(unsigned int minsourcedepth0) = 0;
-    virtual void set_mintargetdepth(unsigned int mintargetdepth0) = 0;
-    virtual void set_maxblocksize(unsigned int maxblocksize)      = 0;
-
-    // Build
-    virtual void build(htool::VirtualGenerator<K> &mat, double* xt, double* xs) = 0;
-    virtual void build(htool::VirtualGenerator<K> &mat, double* xt)             = 0;
 
     virtual ~HMatrixVirt() {};
 };
@@ -120,63 +103,74 @@ public:
 
 template<class K>
 class HMatrixImpl : public HMatrixVirt<K> {
+private:
+    std::shared_ptr<htool::Cluster<double>>  target_cluster;
+    std::shared_ptr<htool::Cluster<double>>  source_cluster;
+    htool::DistributedOperatorFromHMatrix<K> distributed_operator_holder;
+
+
 public:
-    htool::HMatrix<K> H;
-public:
-    HMatrixImpl(std::shared_ptr<htool::VirtualCluster> t, std::shared_ptr<htool::VirtualCluster> s, double epsilon=1e-6 ,double eta=10, char symmetry='N', char UPLO='N',const int& reqrank=-1, MPI_Comm comm=MPI_COMM_WORLD) : H(t,s,epsilon,eta,symmetry,UPLO,reqrank,comm){}
-    const std::map<std::string, std::string>& get_infos() const {return H.get_infos();}
-    void mvprod_global(const K* const in, K* const out,const int& mu=1) const {return H.mvprod_global_to_global(in,out,mu);}
-    int nb_rows() const { return H.nb_rows();}
-    int nb_cols() const { return H.nb_cols();}
-    void cluster_to_target_permutation(const K* const in, K* const out) const {return H.cluster_to_target_permutation(in,out);}
-    void source_to_cluster_permutation(const K* const in, K* const out) const {return H.source_to_cluster_permutation(in,out);}
-    MPI_Comm get_comm() const {return H.get_comm();}
-    int get_rankworld() const {return H.get_rankworld();}
-    int get_sizeworld() const {return H.get_sizeworld();}
-    int get_local_size() const {return H.get_local_size();}
-    int get_local_offset() const {return H.get_local_offset();}
-    const std::vector<htool::SubMatrix<K>*>& get_MyNearFieldMats() const {return H.get_MyNearFieldMats();}
-    const htool::LowRankMatrix<K>& get_MyFarFieldMats(int i) const {return *(H.get_MyFarFieldMats()[i]);}
-    int get_MyFarFieldMats_size() const {return H.get_MyFarFieldMats().size();}
-    const std::vector<htool::SubMatrix<K>*>& get_MyStrictlyDiagNearFieldMats() const {return H.get_MyStrictlyDiagNearFieldMats();}
-    htool::Matrix<K> get_local_dense() const {return H.get_local_dense();}
-    int get_permt(int i) const {return H.get_permt(i);}
-    int get_perms(int i) const {return H.get_perms(i);}
-    char get_symmetry_type() const {return H.get_symmetry_type();}
-    void set_compression(std::shared_ptr<htool::VirtualLowRankGenerator<K>> compressor) {H.set_compression(compressor);};
-
-    // Getters/setters for parameters
-    double get_epsilon() const  {return H.get_epsilon();}                          
-    double get_eta() const    {return H.get_epsilon();}                            
-    int get_minsourcedepth() const {return H.get_minsourcedepth();}              
-    int get_mintargetdepth() const {return H.get_mintargetdepth();}            
-    int get_maxblocksize() const   {return H.get_maxblocksize();}              
-    void set_epsilon(double epsilon0) {H.set_epsilon(epsilon0);}             
-    void set_eta(double eta0)        {H.set_eta(eta0);}                        
-    void set_minsourcedepth(unsigned int minsourcedepth0) {H.set_minsourcedepth(minsourcedepth0);}   
-    void set_mintargetdepth(unsigned int mintargetdepth0) {H.set_mintargetdepth(mintargetdepth0);}   
-    void set_maxblocksize(unsigned int maxblocksize0)   {H.set_maxblocksize(maxblocksize0);}      
-
-    // Build
-    void build(htool::VirtualGenerator<K> &mat, const std::vector<htool::R3> &xt, const std::vector<double> &rt, const std::vector<int> &tabt, const std::vector<double> &gt, const std::vector<htool::R3> &xs, const std::vector<double> &rs, const std::vector<int> &tabs, const std::vector<double> &gs) {H.build(mat,xt,tabt,xs,tabs);}
-
-
-    void build(htool::VirtualGenerator<K> &mat, double* xt, double* xs) {H.build(mat,xt,xs);}
-    void build(htool::VirtualGenerator<K> &mat, double* xt) {H.build(mat,xt);}                       
+    const htool::HMatrix<K>& H;
+    const htool::DistributedOperator<K>& distributed_operator;
+    std::map<std::string, std::string> infos;
+    HMatrixImpl(htool::VirtualGenerator<K> &mat, std::shared_ptr<htool::Cluster<double>> t, std::shared_ptr<htool::Cluster<double>> s,const htool::HMatrixTreeBuilder<K,double>& hmatrix_tree_builder,MPI_Comm comm) : target_cluster(t), source_cluster(s), distributed_operator_holder(mat,*target_cluster,*source_cluster,hmatrix_tree_builder,comm),distributed_operator(distributed_operator_holder.distributed_operator), H(distributed_operator_holder.hmatrix), infos(htool::get_distributed_hmatrix_information(H,distributed_operator.get_comm())){}
+    const std::map<std::string, std::string>& get_infos() const { return infos; }
+    void mvprod_global(const K* const in, K* const out,const int& mu=1) const {return distributed_operator.matrix_product_global_to_global(in,out,mu);}
+    int nb_rows() const { return distributed_operator.get_target_partition().get_global_size();}
+    int nb_cols() const { return distributed_operator.get_source_partition().get_global_size();}
+    void cluster_to_target_permutation(const K* const in, K* const out) const {return htool::cluster_to_user(*target_cluster,in,out);}
+    void source_to_cluster_permutation(const K* const in, K* const out) const {return htool::user_to_cluster(*source_cluster,in,out);}
+    MPI_Comm get_comm() const {return distributed_operator.get_comm();}
+    int get_rankworld() const {int rankworld; MPI_Comm_rank(distributed_operator.get_comm(),&rankworld); return rankworld;}
+    int get_sizeworld() const {int sizeworld; MPI_Comm_size(distributed_operator.get_comm(),&sizeworld); return sizeworld;}
+    int get_local_size() const {int rankworld; MPI_Comm_rank(distributed_operator.get_comm(),&rankworld); return distributed_operator.get_target_partition().get_size_of_partition(rankworld);}
+    int get_local_offset() const {int rankworld; MPI_Comm_rank(distributed_operator.get_comm(),&rankworld); return distributed_operator.get_target_partition().get_offset_of_partition(rankworld);}
+    std::vector<const htool::HMatrix<K>*> get_dense_blocks()const{    
+        std::vector<const htool::HMatrix<K>*> dense_blocks;
+        preorder_tree_traversal(
+        H,
+        [&dense_blocks](const htool::HMatrix<K> &current_hmatrix) {
+            if (current_hmatrix.is_leaf() && current_hmatrix.is_dense()) {
+                dense_blocks.push_back(&current_hmatrix);
+            }
+        });
+        return dense_blocks;}
+    std::vector<const htool::HMatrix<K>*> get_low_rank_blocks()const{    
+        std::vector<const htool::HMatrix<K>*> low_rank_blocks;
+        preorder_tree_traversal(
+        H,
+        [&low_rank_blocks](const htool::HMatrix<K> &current_hmatrix) {
+            if (current_hmatrix.is_leaf() && current_hmatrix.is_low_rank()) {
+                low_rank_blocks.push_back(&current_hmatrix);
+            }
+        });
+        return low_rank_blocks;}
+    std::vector<const htool::HMatrix<K>*> get_diagonal_blocks() const{    
+        std::vector<const htool::HMatrix<K>*> diagonal_blocks;
+        preorder_tree_traversal(
+        H,
+        [&diagonal_blocks](const htool::HMatrix<K> &current_hmatrix) {
+            if (current_hmatrix.is_dense() && current_hmatrix.get_target_cluster().get_offset()==current_hmatrix.get_source_cluster().get_offset()) {
+                diagonal_blocks.push_back(&current_hmatrix);
+            }
+        });
+        return diagonal_blocks;}
+    htool::Matrix<K> get_local_dense() const {htool::Matrix<K> dense(H.nb_rows(),H.nb_cols());htool::copy_to_dense(H,dense.data());return dense;}
+    int get_permt(int i) const {return H.get_target_cluster().get_permutation()[i];}
+    int get_perms(int i) const {return H.get_source_cluster().get_permutation()[i];}
 };
 
 
 struct Data_Bem_Solver
 : public Data_Sparse_Solver {
     double eta;
-    int minclustersize,maxblocksize,mintargetdepth,minsourcedepth;
+    int minclustersize,mintargetdepth,minsourcedepth;
     string compressor, initialclustering;
        
     Data_Bem_Solver()
        : Data_Sparse_Solver(),
        eta(ff_htoolEta),
        minclustersize(ff_htoolMinclustersize),
-       maxblocksize(ff_htoolMaxblocksize),
        mintargetdepth(ff_htoolMintargetdepth),
        minsourcedepth(ff_htoolMinsourcedepth),
        compressor("partialACA"),
@@ -268,7 +262,6 @@ inline void SetEnd_Data_Bem_Solver(Stack stack,Data_Bem_Solver & ds,Expression c
         }
         if (nargs[++kk]) ds.eta = GetAny<double>((*nargs[kk])(stack));
         if (nargs[++kk]) ds.minclustersize = GetAny<int>((*nargs[kk])(stack));
-        if (nargs[++kk]) ds.maxblocksize = GetAny<int>((*nargs[kk])(stack));
         if (nargs[++kk]) ds.mintargetdepth = GetAny<int>((*nargs[kk])(stack));
         if (nargs[++kk]) ds.minsourcedepth = GetAny<int>((*nargs[kk])(stack));
         if (nargs[++kk]) ds.compressor = *GetAny<string*>((*nargs[kk])(stack));
@@ -282,12 +275,13 @@ class HMatrixVirt;
 
 template<class Type, class K>
 KNM<K>* HMatrixVirtToDense(HMatrixVirt<K> **Hmat)
-{ 
+{
     ffassert(Hmat && *Hmat);
     HMatrixVirt<K>& H = **Hmat;
     if(std::is_same<Type, KNM<K>>::value) {
+
         htool::Matrix<K> mdense = H.get_local_dense();
-        KNM<K>* M = new KNM<K>( (long) H.nb_cols(), (long) H.nb_rows() );
+        KNM<K>* M = new KNM<K>( (long) H.nb_rows(), (long) H.nb_cols() );
         for (int i=0; i< H.nb_rows(); i++)
             for (int j=0; j< H.nb_cols(); j++)
                 (*M)(i,j) = 0;
@@ -303,12 +297,20 @@ KNM<K>* HMatrixVirtToDense(HMatrixVirt<K> **Hmat)
 }
 
 template< class FESPACE>
-std::shared_ptr<htool::VirtualCluster> build_clustering(int n, const FESPACE * Uh, const std::vector<double> & p, const Data_Bem_Solver& ds, MPI_Comm comm) {
-    std::shared_ptr<htool::Cluster<htool::PCARegularClustering>> c = std::make_shared<htool::Cluster<htool::PCARegularClustering>>();
+std::shared_ptr<htool::Cluster<double>> build_clustering(int n, const FESPACE * Uh, const std::vector<double> & p, const Data_Bem_Solver& ds, MPI_Comm comm) {
+    htool::ClusterTreeBuilder<double> cluster_builder;
+    std::shared_ptr<htool::Cluster<double>> cluster;
 
-    c->set_minclustersize(ds.minclustersize);
+    cluster_builder.set_direction_computation_strategy(std::make_shared<htool::ComputeLargestExtent<double>>());
+    cluster_builder.set_splitting_strategy(std::make_shared<htool::RegularSplitting<double>>());
+    cluster_builder.set_minclustersize(ds.minclustersize);
+    // std::shared_ptr<htool::Cluster<htool::PCARegularClustering>> c = std::make_shared<htool::Cluster<htool::PCARegularClustering>>();
+
+    int sizeWorld;
+    MPI_Comm_size(comm, &sizeWorld);
+    // c->set_minclustersize(ds.minclustersize);
     if (ds.initialclustering == "" || ds.initialclustering == "pca") {
-        c->build(n,p.data(),2,comm);
+        cluster = std::make_shared<htool::Cluster<double>>(cluster_builder.create_cluster_tree(n,3,p.data(),2,sizeWorld));
     }
     else if (ds.initialclustering == "metis") {
     #if defined(HTOOL_VERSION_GE)
@@ -331,8 +333,7 @@ std::shared_ptr<htool::VirtualCluster> build_clustering(int n, const FESPACE * U
     }
     else
         std::fill(part.begin(),part.end(),0);
-
-    c->build_with_perm(n,p.data(),part.data(),2,comm);
+    cluster = std::make_shared<htool::Cluster<double>>(cluster_builder.create_cluster_tree(n,3,p.data(),2,sizeWorld,part.data()));
     #else
     if (mpirank == 0) std::cerr << "Error: cannot use metis for initial htool clustering ; no metis library" << std::endl;
     ffassert(0);
@@ -350,16 +351,18 @@ std::shared_ptr<htool::VirtualCluster> build_clustering(int n, const FESPACE * U
         if (mpirank == 0) std::cerr << "Error: unknown initial clustering \"" << ds.initialclustering << "\", please use \"pca\" or \"metis\"" << std::endl;
         ffassert(0);
     }
-    return c;
+    return cluster;
 }
 
 template <class R>
 void buildHmat(HMatrixVirt<R>** Hmat, htool::VirtualGenerator<R>* generatorP,const Data_Bem_Solver& data,
-                std::shared_ptr<htool::VirtualCluster> s, std::shared_ptr<htool::VirtualCluster> t,
-                vector<double> &p1,vector<double> &p2,MPI_Comm comm) {
+                std::shared_ptr<htool::Cluster<double>> t, std::shared_ptr<htool::Cluster<double>> s,
+                vector<double> &pt,vector<double> &ps,MPI_Comm comm) {
 
-    *Hmat = new HMatrixImpl<R>(t,s,data.epsilon,data.eta,data.sym?'S':'N',data.sym?'U':'N',-1,comm);
-    std::shared_ptr<htool::VirtualLowRankGenerator<R>> LowRankGenerator = nullptr;
+    int rankWorld;
+    MPI_Comm_rank(comm, &rankWorld);
+    auto hmatrix_builder = htool::HMatrixTreeBuilder<R, double>(*t, *s, data.epsilon, data.eta, data.sym?'S':'N',data.sym?'U':'N', -1,rankWorld,rankWorld);
+    std::shared_ptr<htool::VirtualLowRankGenerator<R,double>> LowRankGenerator = nullptr;
     if (data.compressor=="" || data.compressor == "partialACA")
         LowRankGenerator = std::make_shared<htool::partialACA<R>>();
     else if (data.compressor == "fullACA")
@@ -371,15 +374,12 @@ void buildHmat(HMatrixVirt<R>** Hmat, htool::VirtualGenerator<R>* generatorP,con
         ffassert(0);
     }
 
-    (*Hmat)->set_compression(LowRankGenerator);
-    (*Hmat)->set_maxblocksize(data.maxblocksize);
-    (*Hmat)->set_mintargetdepth(data.mintargetdepth);
-    (*Hmat)->set_minsourcedepth(data.minsourcedepth);
-    // TODO set options
-    if (s == t)
-        (*Hmat)->build(*generatorP,p1.data());
-    else
-        (*Hmat)->build(*generatorP,p1.data(),p2.data());
+    hmatrix_builder.set_low_rank_generator(LowRankGenerator);
+    hmatrix_builder.set_minimal_target_depth(data.mintargetdepth);
+    hmatrix_builder.set_minimal_source_depth(data.minsourcedepth);
+    hmatrix_builder.set_block_tree_consistency(false);
+    HMatrixImpl<R>* test =new HMatrixImpl<R>(*generatorP, t,s,hmatrix_builder,comm);
+    *Hmat = test;
 }
 
 template<class R, class MMesh, class FESpace1, class FESpace2> 
@@ -398,8 +398,8 @@ void creationHMatrixtoBEMForm(const FESpace1 * Uh, const FESpace2 * Vh, const in
     typedef typename std::conditional<SMesh::RdHat::d==1,bemtool::P2_1D,bemtool::P2_2D>::type P2;
 
     // size of the matrix
-    int n=Uh->NbOfDF;
-    int m=Vh->NbOfDF;
+    int m=Uh->NbOfDF;
+    int n=Vh->NbOfDF;
 
     /*
     int VFBEM = typeVFBEM(largs,stack);
@@ -420,8 +420,8 @@ void creationHMatrixtoBEMForm(const FESpace1 * Uh, const FESpace2 * Vh, const in
     Mesh2Bemtool(ThU, node, mesh);
 
 
-    vector<double> p1(3*n);
-    vector<double> p2(3*m);
+    vector<double> pt(3*n);
+    vector<double> ps(3*m);
     Fem2D::R3 pp;
     bemtool::R3 p;
     SRdHat pbs;
@@ -443,26 +443,26 @@ void creationHMatrixtoBEMForm(const FESpace1 * Uh, const FESpace2 * Vh, const in
 
     if (SP2) {
         bemtool::Dof<P2> dof(mesh,true);
-        for (int i=0; i<n; i++) {
+        for (int i=0; i<m; i++) {
             const std::vector<bemtool::N2>& jj = dof.ToElt(i);
             p = dof(jj[0][0])[jj[0][1]];
-            p1[3*i+0] = p[0];
-            p1[3*i+1] = p[1];
-            p1[3*i+2] = p[2];
+            ps[3*i+0] = p[0];
+            ps[3*i+1] = p[1];
+            ps[3*i+2] = p[2];
         }
     }
     else if (SRT0) {
         bemtool::Dof<bemtool::RT0_2D> dof(mesh);
-        for (int i=0; i<n; i++) {
+        for (int i=0; i<m; i++) {
             const std::vector<bemtool::N2>& jj = dof.ToElt(i);
             p = dof(jj[0][0])[jj[0][1]];
-            p1[3*i+0] = p[0];
-            p1[3*i+1] = p[1];
-            p1[3*i+2] = p[2];
+            ps[3*i+0] = p[0];
+            ps[3*i+1] = p[1];
+            ps[3*i+2] = p[2];
         }
     }
     else
-    for (int i=0; i<n; i++) {
+    for (int i=0; i<m; i++) {
         if (SP1)
             pp = ThU.vertices[i];
         else if (SP0)
@@ -471,18 +471,18 @@ void creationHMatrixtoBEMForm(const FESpace1 * Uh, const FESpace2 * Vh, const in
             if (mpirank == 0) std::cerr << "ff-BemTool error: only P0, P1 and P2 discretizations are available for now." << std::endl;
             ffassert(0);
         }
-        p1[3*i+0] = pp.x;
-        p1[3*i+1] = pp.y;
-        p1[3*i+2] = pp.z;
+        ps[3*i+0] = pp.x;
+        ps[3*i+1] = pp.y;
+        ps[3*i+2] = pp.z;
     }
 
-    std::shared_ptr<htool::VirtualCluster> t, s;
-    t = build_clustering(n, Uh, p1, ds, comm);
+    std::shared_ptr<htool::Cluster<double>> t, s;
+    s = build_clustering(m, Uh, ps, ds, comm);
 
     if(!samemesh) {
         if( Vh->TFE[0]->N == 1){
             // case the target FE is scalar
-            for (int i=0; i<m; i++) {
+            for (int i=0; i<n; i++) {
                 if (Vh->MaxNbNodePerElement == TRdHat::d + 1)
                     pp = ThV.vertices[i];
                 else if (Vh->MaxNbNodePerElement == 1)
@@ -491,25 +491,25 @@ void creationHMatrixtoBEMForm(const FESpace1 * Uh, const FESpace2 * Vh, const in
                     if (mpirank == 0) std::cerr << "ff-BemTool error: only P0 and P1 FEspaces are available for reconstructions." << std::endl;
                     ffassert(0);
                 }
-                p2[3*i+0] = pp.x;
-                p2[3*i+1] = pp.y;
-                p2[3*i+2] = pp.z;
+                pt[3*i+0] = pp.x;
+                pt[3*i+1] = pp.y;
+                pt[3*i+2] = pp.z;
             }
         }
         else{
             // hack for Maxwell case to have one Hmatrix to avoid one Hmatrix by direction
             ffassert(SRT0 && SRdHat::d == 2 && VFBEM==2);
 
-            // Dans un espace verctoriel, [P1,P1,P1] pour les targets, on a:
-            //        m correspond au nombre de dof du FEM space
-            // Or dans ce cas, on veut que m = mesh_Target.nv 
+            // Dans un espace vectoriel, [P1,P1,P1] pour les targets, on a:
+            //        n correspond au nombre de dof du FEM space
+            // Or dans ce cas, on veut que n = mesh_Target.nv
             // 
             // ==> on n'a pas besoin de resize les points p2
             int nnn= Vh->TFE[0]->N; // the size of the vector FESpace. For [P1,P1,P1], nnn=3;
 
-            int mDofScalar = m/nnn; // computation of the dof of one component 
+            int nDofScalar = n/nnn; // computation of the dof of one component
 
-            for (int i=0; i<mDofScalar; i++) {
+            for (int i=0; i<nDofScalar; i++) {
                 if (Vh->MaxNbNodePerElement == TRdHat::d + 1)
                     pp = ThV.vertices[i];
                 else if (Vh->MaxNbNodePerElement == 1)
@@ -520,18 +520,18 @@ void creationHMatrixtoBEMForm(const FESpace1 * Uh, const FESpace2 * Vh, const in
                 }
 
                 for(int iii=0; iii<nnn; iii++){
-                    ffassert( nnn*3*i+3*iii+2 < nnn*3*m );
-                    p2[nnn*3*i+3*iii+0] = pp.x;
-                    p2[nnn*3*i+3*iii+1] = pp.y;
-                    p2[nnn*3*i+3*iii+2] = pp.z;
+                    ffassert( nnn*3*i+3*iii+2 < nnn*3*n );
+                    pt[nnn*3*i+3*iii+0] = pp.x;
+                    pt[nnn*3*i+3*iii+1] = pp.y;
+                    pt[nnn*3*i+3*iii+2] = pp.z;
                 }
             }
         }
-        s = build_clustering(m, Vh, p2, ds, comm);
+        t = build_clustering(n, Vh, pt, ds, comm);
     }
     else{
-        p2=p1;
-        s=t;
+        pt=ps;
+        t=s;
     }
 
     // creation of the generator for htool and creation the matrix
@@ -583,7 +583,7 @@ void creationHMatrixtoBEMForm(const FESpace1 * Uh, const FESpace2 * Vh, const in
             ffassert(0);
 
         // build the Hmat
-        buildHmat(Hmat, generator, ds, t, s, p1, p2, comm);
+        buildHmat(Hmat, generator, ds, t, s, pt, ps, comm);
 
         delete generator;
     }
@@ -599,7 +599,7 @@ void creationHMatrixtoBEMForm(const FESpace1 * Uh, const FESpace2 * Vh, const in
         if (Vh->MaxNbNodePerElement == TRdHat::d + 1)
             Mesh2Bemtool(ThV,node_output);
         else if (Vh->MaxNbNodePerElement == 1) {
-            for (int i=0; i<m; i++) {
+            for (int i=0; i<n; i++) {
                 pp = ThV[i](pbt);
                 p[0]=pp.x; p[1]=pp.y; p[2]=pp.z;
                 node_output.setnodes(p);
@@ -628,7 +628,7 @@ void creationHMatrixtoBEMForm(const FESpace1 * Uh, const FESpace2 * Vh, const in
         else
             ffassert(0);
 
-        buildHmat(Hmat, generator, ds, t, s, p1, p2, comm);
+        buildHmat(Hmat, generator, ds, t, s, pt, ps, comm);
 
         delete generator;
     }
@@ -647,3 +647,4 @@ template<> void creationHMatrixtoBEMForm<double, MeshL, FESpaceL, FESpaceL>(cons
                              }
 
 #endif
+;
