@@ -39,7 +39,7 @@ public:
         for(int i=0;i<2;i++) {typeKernel[i]=Bk.typeKernel[i]; wavenum[i]=Bk.wavenum[i]; coeffcombi[i]=Bk.coeffcombi[i]; } } ;
     // alpha * ker
     BemKernel(Stack s,const BemKernel &Bk, Complex alpha) {
-           typeKernel[0]=Bk.typeKernel[0]; wavenum[0]=Bk.wavenum[0]; coeffcombi[0]=alpha; } ;
+        for(int i=0;i<2;i++) {typeKernel[i]=Bk.typeKernel[i]; wavenum[i]=Bk.wavenum[i]; coeffcombi[i]=alpha*Bk.coeffcombi[i]; } } ;
        
     
 private:
@@ -340,6 +340,24 @@ struct Op_addBemKernel {
 
 template<bool INIT,class RR,class AA=RR,class BB=AA>
 struct Op_setBemKernel {
+    using first_argument_type  = AA;
+    using second_argument_type = BB;
+    using result_type          = RR;
+    static RR f(Stack stack, const AA & a,const BB & b)
+    {
+        ffassert(a);
+        const pBemKernel p=new BemKernel(*b);
+
+        if (!INIT && *a)
+            (**a).destroy( );
+        *a = p;
+        return a;
+    }
+};
+
+
+template<bool INIT,class RR,class AA=RR,class BB=AA>
+struct Op_setCombBemKernel {
     using first_argument_type  = AA;
     using second_argument_type = BB;
     using result_type          = RR;
