@@ -909,7 +909,7 @@ void openSVG(const char *filename )
   }
   else
   {
-    cerr << " Err openning SVG file " << fsvg << endl;
+    cerr << " Err opening SVG file " << fsvg << endl;
   }
   return;
 }
