@@ -33,7 +33,7 @@
 #include <fstream>
 #include <iostream>
 #include <cstring>
-#include "libmesh5.h"
+#include "libmeshb7.h"
 #include "ufunction.hpp"
 #include "error.hpp"
 #include "RNM.hpp"
@@ -289,14 +289,18 @@ namespace Fem2D
     
   int MeshL::load(const string & filename)
   {
-    int bin;
-    int ver,inm,dim;
+   int bin;
+   int ver;
+   int64_t inm;
+   int dim;
     int lf=filename.size()+20;
     KN<char>  fileb(lf),filef(lf);
     char *data = new char[filename.size()+1];
     size_t ssize = filename.size()+1;
     char *ptr;
     char *pfile=data;
+    int err=0;
+
     strncpy( data, filename.c_str(),ssize);
     ptr = strstr(data,".mesh");
     if( !ptr ){
@@ -327,14 +331,13 @@ namespace Fem2D
     nEdge=GmfStatKwd(inm,GmfEdges); // segment elements
     nPts=0; // GmfStatKwd(inm,GmfVertices);  // points border element
     this->set(nv,nEdge,nPts);
-       
     if(nEdge == 0) {
       cerr << "  A meshL type must have elements  " << endl;
       ffassert(0);exit(1);}
         
     if(verbosity>1)
       cout << "  -- MeshL(load): "<< (char *) data <<  ", MeshVersionFormatted:= " << ver << ", space dimension:= "<< dim
-	   << ", num Edge elts:= " << nEdge << ", num vertice:= " << nv << " num Points boundaries:= " << nPts << endl;
+	   << ", num Edge elts:= " << nEdge << ", num vertice:= " << nv << " num Points boundaries:= " << nPts << " nerr= " << err << endl;
         
     if(dim  != 3) {
       cerr << "Err dim == " << dim << " !=3 " <<endl;
@@ -366,7 +369,7 @@ namespace Fem2D
         GmfGotoKwd(inm,GmfEdges);
         for(int i=0;i<nEdge;++i) {
             GmfGetLin(inm,GmfEdges,&iv[0],&iv[1],&lab);
-            assert( iv[0]>0 && iv[0]<=nv && iv[1]>0 && iv[1]<=nv);
+            ffassert( iv[0]>0 && iv[0]<=nv && iv[1]>0 && iv[1]<=nv);
             for(int j=0;j<2;++j)
                 if(!vertices[iv[j]-1].lab) {
                     vertices[iv[j]-1].lab=1;
@@ -374,6 +377,11 @@ namespace Fem2D
                 }
             for (int j=0;j<2;++j) iv[j]--;
                 elements[i].set(vertices,iv,lab);
+            if( abs(elements[i].mesure())<1e-30) {
+                if(verbosity && err < 10)
+                cout << " Err load meshL zero element "<< i << " vertices:  " << iv[0] << " " << iv[1] << endl;
+                err++;
+            }
           mes += elements[i].mesure();
         }
         if(kmv&& verbosity>1) cout << "    Aucun label Hack (FH)  ??? => 1 sur les triangle frontiere "<<endl;
@@ -383,9 +391,14 @@ namespace Fem2D
         GmfGotoKwd(inm,GmfEdges);
         for(int i=0;i<nEdge;++i) {
             GmfGetLin(inm,GmfEdges,&iv[0],&iv[1],&lab);
-            assert( iv[0]>0 && iv[0]<=nv && iv[1]>0 && iv[1]<=nv);
+            ffassert( iv[0]>0 && iv[0]<=nv && iv[1]>0 && iv[1]<=nv);
             for (int j=0;j<2;++j) iv[j]--;
                 elements[i].set(this->vertices,iv,lab);
+            if( abs(elements[i].mesure())<1e-30) {
+                if(verbosity && err < 10)
+                cout << " Err load meshL zero element "<< i << " vertices:  " << iv[0] << " " << iv[1] << endl;
+                err++;
+            }
             mes += elements[i].mesure();
         }
     }
@@ -394,7 +407,7 @@ namespace Fem2D
     GmfGotoKwd(inm,GmfVertices);
     for(int i=0;i<nPts;++i) {
         GmfGetLin(inm,GmfVertices,&iv[0],&lab);
-        assert( iv[0]>0 && iv[0]<=nv);
+        ffassert( iv[0]>0 && iv[0]<=nv);
         borderelements[i].set(this->vertices,iv,lab);
         mesb += this->borderelements[i].mesure();
     }
@@ -402,10 +415,15 @@ namespace Fem2D
         
     if(verbosity>1)
         cout << "  -- MeshL(load): "<< (char *) data <<  ", MeshVersionFormatted:= " << ver << ", space dimension:= "<< dim
-	    << ", Edges elts:= " << nt << ", num vertice:= " << nv << ", num Points boundaries:= " << nbe << endl;
-        
+	    << ", Edges elts:= " << nt << ", num vertice:= " << nv << ", num Points boundaries:= " << nbe << endl; 
     GmfCloseMesh(inm);
     delete[] data;
+    if(err) {
+           cerr << " readmeshL: wrong mesh : number of zero seg : "<< err << endl;
+            ErrorExec("readmeshL mesh with zero element ",1);
+          return -1;
+        }
+
     return 0; // OK
   }
     
@@ -519,7 +537,7 @@ namespace Fem2D
             for(int jj=0; jj <2; jj++)
                 iv[jj] = this->operator()(K[jj]);
             if(verbosity>2)
-                cout << "EdgeL: " << k << " lenght "<<  this->elements[k].mesure() << endl;
+                cout << "EdgeL: " << k << " length "<<  this->elements[k].mesure() << endl;
             if(verbosity>2) cout << " A triangleS with a very small edge was created " << endl;
             return 1;
         }
@@ -589,7 +607,8 @@ namespace Fem2D
           {
               double l[2];
               int loutside;// 0 inside, 1 out close, 2, out fare, , -1 inside
-              int itt =gdfb->Find(P,l,loutside);
+              long old = tstart ? tstart-elements  : -1;
+              int itt =gdfb->Find(P,l,loutside,old);
               outside=loutside;
               Phat=R1(l[1]);
               Element &K=(this->elements)[itt];
@@ -687,12 +706,13 @@ namespace Fem2D
  
   int MeshL::Save(const string & filename) const
   {
-    int ver = GmfDouble, outm;
+      int ver = GmfDouble;
+      int64_t outm;
     if ( !(outm = GmfOpenMesh(filename.c_str(),GmfWrite,ver,3)) ) {
         cerr <<"  -- MeshL**::Save  UNABLE TO OPEN  :"<< filename << endl;
         return(1);
     }
-    float fx,fy,fz;
+    double fx,fy,fz;
     // write vertice (meshL)
     GmfSetKwd(outm,GmfVertices,nv);
     for (int k=0; k<nv; k++) {
@@ -771,14 +791,14 @@ namespace Fem2D
                            cout << "Element num: " << i << " N " << E << " Element adjacent num: " << it << " E_adj " << E_adj << " angle between N N_adj = " << pdt << " " << (E,E_adj) << endl;
                         
                        if(pdt >= angle) {
-                           if(verbosity>15)
+                           if(verbosity>1)
                                cout << " the border point " <<nbeL <<": [" << iv[0] << " " << "] is a boundary with the angular criteria " << angle << " " << vertices[iv[0]].lab << endl;
                            int lab = vertices[iv[0]].lab; //min(K.lab, K_adj.lab);
                            be(nbeL).set(vertices,iv,lab);
                            pointI.add(key, nbeL++);
                        }
                        else if(labeledBoundary && K.lab != K_adj.lab ) {
-                           if(verbosity>15)
+                           if(verbosity>1)
                              cout << " the border point " <<nbeL <<": [" << iv[0] << " " << "] is a boundary with labeled boundary criteria" << vertices[iv[0]].lab <<endl;
             
                            be(nbeL).set(vertices,iv,vertices[iv[0]].lab);//K.lab);
